"use strict";
/**
 * @license
 * Copyright (c) 2017 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
Object.defineProperty(exports, "__esModule", { value: true });
const base_converter_1 = require("./base-converter");
const document_converter_1 = require("./document-converter");
const _isInBowerRegex = /(\b|\/|\\)(bower_components)(\/|\\)/;
const _isInNpmRegex = /(\b|\/|\\)(node_modules)(\/|\\)/;
const isNotExternal = (d) => !_isInBowerRegex.test(d.url) && !_isInNpmRegex.test(d.url);
/**
 * Converts an entire Analysis object.
 */
class AnalysisConverter extends base_converter_1.BaseConverter {
    constructor(analysis, options) {
        super(analysis, options);
        this.packageName = options.packageName;
        this.packageType = options.packageType || 'element';
        const includes = this.includes;
        for (const mainfile of options.mainFiles || []) {
            includes.add(mainfile);
        }
    }
    getDocumentConverter(document, visited) {
        return new document_converter_1.DocumentConverter(this, document, this.packageName, this.packageType, visited);
    }
    filter(document) {
        return isNotExternal(document) && !!document.url;
    }
}
exports.AnalysisConverter = AnalysisConverter;
//# sourceMappingURL=analysis-converter.js.map