"use strict";
/**
 * @license
 * Copyright (c) 2017 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const url_converter_1 = require("../url-converter");
suite('src/url-converter', () => {
    suite('convertDocumentUrl()', () => {
        test('converts a local html url to expected js url', () => {
            chai_1.assert.equal(url_converter_1.convertHtmlDocumentUrl('foo.html'), './foo.js');
            chai_1.assert.equal(url_converter_1.convertHtmlDocumentUrl('foo/foo.html'), './foo/foo.js');
        });
        test('converts a bower_components/ (external) html url to expected js url', () => {
            chai_1.assert.equal(url_converter_1.convertHtmlDocumentUrl('bower_components/polymer/polymer.html'), './node_modules/@polymer/polymer/polymer.js');
            chai_1.assert.equal(url_converter_1.convertHtmlDocumentUrl('bower_components/paper-item/src/paper-item.html'), './node_modules/@polymer/paper-item/src/paper-item.js');
            chai_1.assert.equal(url_converter_1.convertHtmlDocumentUrl('bower_components/promise-polyfill/promise-polyfill.html'), './node_modules/@polymer/promise-polyfill/promise-polyfill.js');
        });
        test('handles special whitelisted url conversions', () => {
            chai_1.assert.equal(url_converter_1.convertHtmlDocumentUrl('bower_components/shadycss/apply-shim.html'), './node_modules/@webcomponents/shadycss/entrypoints/apply-shim.js');
            chai_1.assert.equal(url_converter_1.convertHtmlDocumentUrl('bower_components/shadycss/custom-style-interface.html'), './node_modules/@webcomponents/shadycss/entrypoints/custom-style-interface.js');
        });
    });
    suite('getRelativeUrl()', () => {
        test('handles two root urls relative to the same directory', () => {
            chai_1.assert.equal(url_converter_1.getRelativeUrl('./foo.js', './bar.js'), './bar.js');
            chai_1.assert.equal(url_converter_1.getRelativeUrl('./foo/foo.js', './bar.js'), '../bar.js');
            chai_1.assert.equal(url_converter_1.getRelativeUrl('./foo/foo.js', './bar/bar.js'), '../bar/bar.js');
        });
        test('explicitly does not handle sibling/parent urls', () => {
            chai_1.assert.throws(() => {
                url_converter_1.getRelativeUrl('../foo.js', './bar.js');
            }, 'paths relative to package root expected (actual: from="../foo.js", to="./bar.js")');
        });
    });
});
//# sourceMappingURL=url-converter_test.js.map