import '../polymer/polymer.js';
import '../marked-element/marked-element.js';
import './iron-doc-viewer-styles.js';
import { Polymer } from '../polymer/lib/legacy/polymer-fn.js';
/**
 * Renders documentation describing a function or method.
 */
Polymer({
  _template: `
    <style include="iron-doc-viewer-styles"></style>
    <style>
      :host {
        display: block;
      }

      .paramName {
        color: var(--iron-doc-accent-color-internal);
      }

      .paramType, .returnType {
        color: #666;
      }

      #paramList .paramName {
        color: #666;
        margin-right: 5px;
      }

      #paramList marked-element,
      #paramList .markdown-html,
      #paramList .markdown-html p {
        display: inline-block;
        margin: 0;
      }

      .static {
        color: #666;
      }

      .privacy {
        color: #666;
      }
    </style>

    <code id="signature">
      <span class="privacy">[[_privacy]]</span>
      <span class="static" hidden\$="[[!static]]">static</span>

      <a href\$="#[[anchorId]]" id="funcName" class="name deeplink">
        [[descriptor.name]]<!--
   --></a><!--
   -->(<span class="funcParams"><!--
     --><template is="dom-repeat" items="[[descriptor.params]]" as="p"><!--
       --><span hidden\$="[[!index]]">, </span><!--
       --><span class="paramName">[[p.name]]</span><!--
       --><span hidden\$="[[!p.type]]">:
            <span class="paramType">[[p.type]]</span><!--
       --></span><!--
     --></template><!--
   --></span>)<!--
   --><span hidden\$="[[!descriptor.return]]">:
        <span class="returnType">[[descriptor.return.type]]</span>
      </span>
    </code>

    <p hidden\$="[[!descriptor.inheritedFrom]]" class="inheritedFrom">
      Inherited from <code>[[descriptor.inheritedFrom]]</code>
    </p>

    <marked-element sanitize="" markdown="[[descriptor.description]]" hidden\$="[[!descriptor.description]]">
      <div slot="markdown-html" class="markdown-html"></div>
    </marked-element>

    <ul id="paramList" hidden\$="[[!_showParamList]]">
      <template is="dom-repeat" items="[[descriptor.params]]">
        <li>
          <code class="paramName">[[item.name]]</code>
          <marked-element sanitize="" markdown="[[item.description]]" hidden\$="[[!item.description]]">
           <div slot="markdown-html" class="markdown-html"></div>
         </marked-element>
        </li>
      </template>
    </ul>
`,

  is: 'iron-doc-function',

  properties: {
    anchorId: {
      type: String,
      reflectToAttribute: true,
      value: ''
    },

    static: {
      type: Boolean,
      value: false
    },

    _privacy: {
      type: String,
      computed: '_computePrivacy(descriptor)'
    },

    _showParamList: {
      type: Boolean,
      computed: '_computeShowParamList(descriptor)'
    }
  },

  _computePrivacy: function(descriptor) {
    if (!descriptor || !descriptor.privacy
        || descriptor.privacy === 'public') {
      return '';
    }
    return descriptor.privacy;
  },

  _computeShowParamList: function(descriptor) {
    if (descriptor && descriptor.params) {
      for (var i = 0; i < descriptor.params.length; i++) {
        if (descriptor.params[i].description) {
          return true;
        }
      }
    }
    return false;
  }
});
