import '../polymer/polymer.js';
import { IronControlState } from '../iron-behaviors/iron-control-state.js';
import '../iron-collapse/iron-collapse.js';
import './paper-menu-shared-styles.js';
import { Polymer } from '../polymer/lib/legacy/polymer-fn.js';
import { dom } from '../polymer/lib/legacy/polymer.dom.js';

Polymer({
  _template: `
    <style include="paper-menu-shared-styles"></style>

    <div class="selectable-content" on-tap="_onTap">
      <content id="trigger" select=".menu-trigger"></content>
    </div>
    <iron-collapse id="collapse" opened="{{opened}}">
      <content id="content" select=".menu-content"></content>
    </iron-collapse>
`,

  is: 'paper-submenu',

  properties: {
    /**
     * Fired when the submenu is opened.
     *
     * @event paper-submenu-open
     */

    /**
     * Fired when the submenu is closed.
     *
     * @event paper-submenu-close
     */

    /**
     * Set opened to true to show the collapse element and to false to hide it.
     *
     * @attribute opened
     */
    opened: {
      type: Boolean,
      value: false,
      notify: true,
      observer: '_openedChanged'
    }
  },

  behaviors: [
    IronControlState
  ],

  listeners: {
    'focus': '_onFocus'
  },

  get __parent() {
    return dom(this).parentNode;
  },

  get __trigger() {
    return dom(this.$.trigger).getDistributedNodes()[0];
  },

  get __content() {
    return dom(this.$.content).getDistributedNodes()[0];
  },

  attached: function() {
    this.listen(this.__parent, 'iron-activate', '_onParentIronActivate');
  },

  detached: function() {
    this.unlisten(this.__parent, 'iron-activate', '_onParentIronActivate');
  },

  /**
   * Expand the submenu content.
   */
  open: function() {
    if (!this.disabled) {
      this.opened = true;
    }
  },

  /**
   * Collapse the submenu content.
   */
  close: function() {
    this.opened = false;
  },

  /**
   * Toggle the submenu.
   */
  toggle: function() {
    if (this.opened) {
      this.close();
    } else {
      this.open();
    }
  },

  /**
   * A handler that is called when the trigger is tapped.
   */
  _onTap: function(e) {
    if (!this.disabled) {
      this.toggle();
    }
  },

  /**
   * Toggles the submenu content when the trigger is tapped.
   */
  _openedChanged: function(opened, oldOpened) {
    if (opened) {
      this.__trigger && this.__trigger.classList.add('iron-selected');
      this.__content && this.__content.focus();
      this.fire('paper-submenu-open');
    } else if (oldOpened != null) {
      this.__trigger && this.__trigger.classList.remove('iron-selected');
      this.fire('paper-submenu-close');
    }
  },

  /**
   * A handler that is called when `iron-activate` is fired.
   *
   * @param {CustomEvent} event An `iron-activate` event.
   */
  _onParentIronActivate: function(event) {
    var parent = this.__parent;
    if (dom(event).localTarget === parent) {
      // The activated item can either be this submenu, in which case it
      // should be expanded, or any of the other sibling submenus, in which
      // case this submenu should be collapsed.
      if (event.detail.item !== this && !parent.multi) {
        this.close();
      }
    }
  },

  /**
   * If the dropdown is open when disabled becomes true, close the
   * dropdown.
   *
   * @param {boolean} disabled True if disabled, otherwise false.
   */
  _disabledChanged: function(disabled) {
    IronControlState._disabledChanged.apply(this, arguments);
    if (disabled && this.opened) {
      this.close();
    }
  },

  /**
   * Handler that is called when the menu receives focus.
   *
   * @param {FocusEvent} event A focus event.
   */
  _onFocus: function(event) {
    this.__trigger && this.__trigger.focus();
  }
});
