import '../polymer/polymer.js';
import '../iron-a11y-keys-behavior/iron-a11y-keys-behavior.js';
import './paper-text-field-shared-styles.js';
import { PaperTextFieldBehavior } from './paper-text-field-behavior.js';
import { Polymer } from '../polymer/lib/legacy/polymer-fn.js';
Polymer({
  _template: `
    <style include="paper-text-field-shared-styles">
      /* Prevent new lines from being entered or pasted */
      #input {
        min-height: 20px; /* FF doesn't seem to have a default */
        z-index: 1;
        white-space: nowrap;
        overflow: hidden;
      }

      /* The ::content rule is needed because in the shady DOM, the
      brs are added by the browser and not style-scoped correctly. */
      #input br, #input ::content br  {
        display: none;
      }

      #input *, #input ::content * {
        display: inline;
        white-space: nowrap;
      }
    </style>

    <div class="container">
      <label hidden\$="[[!label]]" class\$="[[_computeLabelClass(noFloatLabel,alwaysFloatLabel,hasContent)]]">[[label]]</label>
      <div id="input" contenteditable="true" tabindex="-1"></div>
    </div>
`,

  is: 'paper-text-field',

  hostAttributes: {
    'tabindex': 0,
    'role': 'textbox'
  },

  keyBindings: {
    'meta+b:keydown': '_preventRichTextFormatting',
    'meta+i:keydown': '_preventRichTextFormatting',
    'ctrl+b:keydown': '_preventRichTextFormatting',
    'ctrl+i:keydown': '_preventRichTextFormatting'
  },

  behaviors: [
    PaperTextFieldBehavior,
  ],

  _setUpListeners: function() {
    this.inputElement.addEventListener('paste', this._onPaste.bind(this));
    this.inputElement.addEventListener('input', this._onInput.bind(this));

    // Never stop being different, IE.
    this.inputElement.addEventListener('textinput', this._onInput.bind(this));
  },

  /**
   * Called after a paste event inside the input element.
   */
  _onPaste: function(event) {
    var pastedText;

    if (window.clipboardData && window.clipboardData.getData) { // IE
      pastedText = window.clipboardData.getData('Text');
    } else if (event.clipboardData && event.clipboardData.getData) {
      pastedText = event.clipboardData.getData('text/plain');
    }

    // IE10 and 11 don't support `insertText`, but support `paste`, which
    // doesn't persist formatting. Modern browsers that support `insertText`
    // also persist the formatting on `paste`, so they should use that.
    if (document.queryCommandSupported('insertText')) {
      document.execCommand('insertText', false, pastedText);
    } else if (document.queryCommandSupported('paste')) {
      document.execCommand('paste', false, pastedText);
    } else {
      return;
    }

    // Allow the pasted text and update the labels.
    event.preventDefault();
    this._onInput();
  },

  /**
   * Called on keydown, to prevent rich formatting of the input.
   */
  _preventRichTextFormatting: function() {
    // Prevent Ctrl+B and Ctrl+I for bolding and italics.
    event.preventDefault();
  }
});
